/* 
   jsKana version 0.2

   Copyright (C) Mateusz Turcza <mturcza@antares.astrouw.edu.pl>
   
   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, 
   MA 02110-1301, USA.
 
*/

/* 
  ChangeLog:

  March 12, 2006
  jsKana version 0.2:
  - Added handling of transcription systems (Hepburn, Japanese gov)
  - Fixed few minor bugs in input parsing
  - Subtle changes in initial parsing with regular expressions
  - Changed romaji display (square brackets removed)
  - Fixed bug from which Netscape 6 couldn't handle radio buttons properly
  
  March 11,2006
  jsKana version 0.1 released

  March 10,2006
  Start of jsKana project development
  
*/

// layout -------------------------------------------------------------

var tStyle = 'border:1px solid gray; background:url(jsk-bg.png) #B4B4B4;'
var iStyle = 'height:160px; border:1px inset #4E4E4E; background:#FAFAFA; overflow:auto; padding:4px; width:600px; '
var oStyle = 'height:160px; border:1px inset #4E4E4E; background:#FAFAFA; overflow:auto; padding:4px; display:block;'
var dStyle = 'border:1px #B4B4B4 inset; background:url(jsk-bg.png) #B4B4B4;'

write("<table border=\"0\" cellpadding=\"3\" cellspacing=\"0\" style=\""+tStyle+"\">" +
      "  <tr>" +
      "    <td align=\"right\" valign=\"top\">Input</td>" +
      "    <td><textarea id=\"_kIn\" style=\""+iStyle+"\" onKeyUp=\"kWrt();\"></textarea></td>" +
      "    <td colspan=\"2\">&nbsp;</td>" +
      "  </tr>" +
      "  <tr>" +
      "    <td align=\"right\" valign=\"top\">Output</td>" +
      "    <td><div id=\"_kOut\" style=\""+oStyle+"\"></td>" +
      "    <td valign=\"top\">" +
      "<table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"0\" style=\""+dStyle+"\">"+
      "  <tr>" +
      "    <td>Display:</td>" +
      "  </tr>" +
      "  <tr>" +
      "    <td>" +
      "      <form>" +
      "      <input type=\"radio\" name=\"_kTy\" onClick=\"kWrt();\"  checked=\"checked\"/> kana<br/>" +
      "      <input type=\"radio\" name=\"_kTy\" onClick=\"kWrt();\"/> romaji<br/>" + 
      "      </form>" +
      "    </td>" +
      "  </tr>" +
      "</table>" +
      "<div id=\"_kTrDiv\" style=\"visibility:hidden;margin-top:10px\">" +     
      "<table width=\"100%\" border=\"0\" cellpadding=\"3\" cellspacing=\"0\" style=\""+dStyle+"\">"+  
      "  <tr>" +
      "    <td>Transcription:</td>" +
      "  </tr>" +       
      "  <tr>" +
      "    <td nowrap=\"nowrap\">" +
      "    <form>" +
      "    <input type=\"radio\" name=\"_kTr\" onClick=\"kWrt();\" checked=\"checked\"/> Hepburn<br/>" +
      "    <input type=\"radio\" name=\"_kTr\" onClick=\"kWrt();\"/> Japanese gov<br/>" +
      "    </form>" + 
      "    </td>" +
      "  </tr>" + 
      "</table>" + 
      "</div>" +
      "    </td>" +
      "  </tr>" + 
      "</table>")
 
// functions ----------------------------------------------------------

function write(s) { document.write(s) }

function getObject(id) {
  return document.all ? document.all[id] : document.getElementById(id)
}

Array.prototype.keyExists = function(val) {
  for(var i in this) {
    if(val==i) return true
  }
  return false
}

function rx(s,rexp) {
// rx ( string , regular_expression )
  return s.search(rexp)!=-1 ? true : false 
}

function err(s) { return '<b>Error:</b> '+s+'<br/>' }

// --------------------------------------------------------------------

function kWrt() {
  getObject('_kOut').innerHTML = kConv(getObject('_kIn').value)
}

function kMode(kanaChk) {
// kanaChk - if kana option is selected
  getObject('_kTrDiv').style.visibility = kanaChk ? 'hidden' : 'visible'
  return kanaChk
}

function kConv(str) {
  if(typeof kana   == 'undefined') return err('Unable to load <i>kana[]</i> array.')
  if(typeof symbol == 'undefined') return err('Unable to load <i>symbol[]</i> array.')

  // display mode: true - kana, false - romaji
  var dm = kMode(document.getElementsByName('_kTy')[0].checked)
  // transcription mode: true - hepburn, false - japanese government
  var tm = document.getElementsByName('_kTr')[0].checked ? true : false
	
  // initial replaces	
  str = str.replace(/(l)/g,"r")
  str = str.replace(/(L)/g,"R")
  
  str = str.replace(/([csCS])[hHyY]([aueoAUEO])/g,"$1h$2") // cya cye => cha che
 
  str = str.replace(/[yY]([iI])/g,"$1")  // yi->i

  str = str.replace(/([csCS])[iI]/g,"$1hi") // replace remaining ci, si => chi shi
  
  str = str.replace(/ty([aueo])/g,'ch$1') // tya -> cha
  str = str.replace(/[tT]c/g    ,"cc")   // tcha -> ccha 

  str = str.replace(/ti/g   ,"chi")	
  str = str.replace(/T[iI]/g,"Chi")
  
  str = str.replace(/zi/g   ,"ji")
  str = str.replace(/Z[iI]/g,"Ji")

  str = str.replace(/[jz]zy/g ,"jjy") // zzya -> jjy 
  str = str.replace(/[JZ][zZ][yY]/g  ,"Jjy")

  str = str.replace(/[jz]y([auo]){1}/g      ,"j$1")
  str = str.replace(/[JZ][yY]([auoAUO]){1}/g,"J$1")
  
  str = str.replace(/([^csCS])hu/g   ,'$1fu') // hu-> fu & omit chu shu
  str = str.replace(/([^csCS])H[uU]/g,'$1Fu') // 

  str = str.replace(/cu|tu|tzu/g     ,"tsu")
  str = str.replace(/[C|T][zZ]?[uU]/g,"Tsu")

  str = str.replace(/du|dzu/g    ,"du")
  str = str.replace(/D[zZ]?[uU]/g,"Du")

  var kstr = ''
  for(i=0;i<str.length;i++) {
    // 0 - hiragana, 1 - katakana
    var ty = str.charAt(i)==str.charAt(i).toLowerCase() ? 0 : 1 
    var c0 = str.charAt(i).toLowerCase()
    var c1 = str.charAt(i+1).toLowerCase()
    var c2 = str.charAt(i+2).toLowerCase()
    var c3 = c0+c1+c2

    if(rx(c0,/[aiueo]/)) {
      kstr += dm ? kana[c0][ty] : c0//'['+c0+']' 
    }
    else if(rx(c0,/^[kgszjtcdmhfbpnrwyv]/)) {
      if(c0==c1 && c0!='n' && (c1!='d' && c2!='y')) { // repeats 
        // no dya dye etc., so there is no ddya, ddye etc.
        var tr = c0
        if(!tm) { // japanese gov
          if(c1=='c') tr='t'
          else if(c1=='j') tr='z'
          else if(c1+c2=='fu') tr='h'
        }
        kstr += dm ? kana['_tsu'][ty] : tr //'[_tsu('+tr+')]'
      } // end doubles
      else if(rx(c3,/^[kgszjtcdmhfbpnrwyv](y|h)?[aiueo]{1,}/)) { // triples & pairs
        if(rx(c3,/^[kgjnhbpmr]y[auoe]/)) { // kya,kyu,kye,kyo etc.
          var cn = kana[(c2=='e'?'_e':('_y'+c2))][ty]
          kstr += dm ? kana[c0+'i'][ty] + cn : c3//'['+c3+']'
          i=i+2
        }
        else if(rx(c3,/^[cs]h[aiuoe]/)) { // cha, chu, che, cho etc.
          var cn = c2=='i'?'':kana[(c2=='e'?'_e':('_y'+c2))][ty]
          var tr = tm ? c3 : c3.replace('c','t').replace('hi','i').replace('h','y')
          kstr += dm ? kana[c0+'hi'][ty]+cn : tr//'['+tr+']'
          i=i+2
        }
        else if(rx(c0+c1,/^[kgszjtdnhbpmyrw][aiueo]/)) { // ka, ki, ku, ke, ko 
          if(c0+c1=='wu') continue // there is no char for 'wu'
          var cn = ''
          var ci = c0+c1
          var tr = c0+c1
          if(c0=='j') { 
            if(c1!='i') { // ja,je -> ji _ya , ji _e
              ci = c0+'i'
              cn = c1!='e'?kana['_y'+c1][ty]:kana['_e'][ty]
              tr = tm ? tr : tr.replace('j','zy')
            }
            else { // ji
              tr = tm ? tr : tr.replace('j','z')
            }
          }
          else if(c0=='y' && c1=='e') { // ye
            ci = 'i' // ye -> i_e
            cn = kana['_e'][ty]
          }
          kstr += dm ? kana[ci][ty]+cn : tr//'['+tr+']'
          i++
        }
        else if(rx(c0+c1,/f[aiueo]/)) { // f
          var cn = c1!='u' ? kana['_'+c1][ty] : ''
          var tr = !tm && c1=='u' ? 'hu' : 'f'+c1 // fu -> hu (japanese gov transc.)
          kstr += dm ? kana['fu'][ty] + cn : tr//'['+tr+']'
          i++
        }
        else if(rx(c0+c1,/v[aiueo]/)) { // v - katakana only
          var cn = c1!='u' ? kana['_'+c1][1] : ''
          kstr += dm ? kana['vu'][1] + cn : c0+c1//'['+c0+c1+']'
          i++
        }
      } // end triples & pairs
      else if(rx(c3,/tsu/)) { // tsu
        kstr += dm ? kana[c3][ty] : (tm ? 'tsu' : 'tu')
        i=i+2
      }  // end tsu
      else if(c0=='n' || rx(c0+c1,/m[bp]/)) { // n
        var tr = !tm && rx(c1,/[bp]/) ? 'm' : 'n'
        kstr += dm ? kana['n'][ty] : tr//'['+tr+']'
        i=i+(c1=="'"?1:0)
      } // end n
    }
    else if(c0=='_' && rx(c1+c2,/(^[ky]?[aiueo])|ts/)) {
      ty = str.charAt(i+1)==str.charAt(i+1).toLowerCase() ? 0 : 1 // get type from secont char
      var cn = null
      var cs = 2
      var tr = c1+c2
      if(rx(c1,/[aiueo]/)) {
        cn = kana['_'+c1][ty]
        cs = 1 // total length is 1
        c2=''
      }
      else if(c1+c2=='ka')               cn = kana['_ka'][ty]
      else if(c1=='y' && rx(c2,/[auo]/)) cn = kana['_y'+c2][ty]
      else if(c1+c2+str.charAt(i+3).toLowerCase()=='tsu') {
        cn = kana['_tsu'][ty]
        tr = tm ? 'tsu' : 'tu'
        cs = 3 // total length of character is 4 (_tsu)
      }
      if(cn!=null) {
        kstr+= dm ? cn : tr//'[_'+tr+']'
        i=i+cs
      }
    }
    else if(symbol.keyExists(c0)){
      kstr += dm ? symbol[c0] : (c0=='\n'?'<br/>':c0)
    }
  }
  return kstr
}

// EOF ----------------------------------------------------------------